@@ -1,20 +1,20 @@
 use std::{
     convert::TryInto,
     ffi::{CStr, CString, OsString},
-    path::{Path, PathBuf},
+    path::{Path, PathBuf}, ptr::NonNull,
 };
 
 use crate::{
-    error::{FindModuleByPathError, GetLocalProcedureError, Win32OrNulError},
+    error::{GetLocalProcedureError, IoOrNulError},
     utils::WinPathBuf,
     ProcessRef,
 };
 use path_absolutize::Absolutize;
-use rust_win32error::Win32Error;
 use widestring::{U16CStr, U16CString};
+use get_last_error::Win32Error;
 use winapi::{
     shared::{
-        minwindef::{__some_function, HMODULE},
+        minwindef::{__some_function, HMODULE, HINSTANCE__},
         winerror::ERROR_MOD_NOT_FOUND,
     },
     um::{
@@ -32,7 +32,7 @@ pub type ModuleHandle = HMODULE;
 /// A loaded module of a running process.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub struct ProcessModule<'a> {
-    handle: ModuleHandle,
+    handle: NonNull<HINSTANCE__>,
     process: ProcessRef<'a>,
 }
 
@@ -41,15 +41,16 @@ impl<'a> ProcessModule<'a> {
     ///
     /// # Safety
     /// The caller must guarantee that the given handle is valid and that the module is loaded into the given process. (and stays that way while interacting).
-    pub const unsafe fn new(handle: ModuleHandle, process: ProcessRef<'a>) -> Self {
+    pub const unsafe fn new_unchecked(handle: ModuleHandle, process: ProcessRef<'a>) -> Self {
+        let handle = unsafe { NonNull::new_unchecked(handle) };
         Self { handle, process }
     }
     /// Contructs a new instance from the given module handle loaded in the current process.
     ///
     /// # Safety
     /// The caller must guarantee that the given handle is valid and that it is loaded into the current process. (and stays that way while interacting).
-    pub unsafe fn new_local(handle: ModuleHandle) -> Self {
-        unsafe { Self::new(handle, ProcessRef::current()) }
+    pub unsafe fn new_local_unchecked(handle: ModuleHandle) -> Self {
+        unsafe { Self::new_unchecked(handle, ProcessRef::current()) }
     }
 
     /// Searches for a module with the given name or path in the given process (the current one if [`None`] was specified).
@@ -57,12 +58,12 @@ impl<'a> ProcessModule<'a> {
     pub fn find(
         module_name_or_path: impl AsRef<Path>,
         process: ProcessRef<'a>,
-    ) -> Result<Option<Self>, FindModuleByPathError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         let module_name_or_path = module_name_or_path.as_ref();
         if module_name_or_path.has_root() {
             Self::find_by_path(module_name_or_path, process)
         } else {
-            Self::find_by_name(module_name_or_path, process).map_err(|e| e.into())
+            Self::find_by_name(module_name_or_path, process)
         }
     }
     /// Searches for a module with the given name in the given process (the current one if [`None`] was specified).
@@ -70,7 +71,7 @@ impl<'a> ProcessModule<'a> {
     pub fn find_by_name(
         module_name: impl AsRef<Path>,
         process: ProcessRef<'a>,
-    ) -> Result<Option<Self>, Win32OrNulError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         if process.is_current() {
             Self::find_local_by_name(module_name)
         } else {
@@ -82,7 +83,7 @@ impl<'a> ProcessModule<'a> {
     pub fn find_by_path(
         module_path: impl AsRef<Path>,
         process: ProcessRef<'a>,
-    ) -> Result<Option<Self>, FindModuleByPathError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         if process.is_current() {
             Self::find_local_by_path(module_path)
         } else {
@@ -94,44 +95,44 @@ impl<'a> ProcessModule<'a> {
     /// If the extension is omitted, the default library extension `.dll` is appended.
     pub fn find_local(
         module_name_or_path: impl AsRef<Path>,
-    ) -> Result<Option<Self>, FindModuleByPathError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         Self::find(module_name_or_path, ProcessRef::current())
     }
     /// Searches for a module with the given name in the current process.
     /// If the extension is omitted, the default library extension `.dll` is appended.
     pub fn find_local_by_name(
         module_name: impl AsRef<Path>,
-    ) -> Result<Option<Self>, Win32OrNulError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         Self::_find_local_by_name_or_abs_path(module_name)
     }
     /// Searches for a module with the given path in the current process.
     /// If the extension is omitted, the default library extension `.dll` is appended.
     pub fn find_local_by_path(
         module_path: impl AsRef<Path>,
-    ) -> Result<Option<Self>, FindModuleByPathError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         let absolute_path = module_path.as_ref().absolutize()?;
-        Self::_find_local_by_name_or_abs_path(absolute_path).map_err(|e| e.into())
+        Self::_find_local_by_name_or_abs_path(absolute_path)
     }
     pub(crate) fn _find_local_by_name_or_abs_path(
         module: impl AsRef<Path>,
-    ) -> Result<Option<Self>, Win32OrNulError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         let wide_string = U16CString::from_os_str(module.as_ref().as_os_str())?;
         Self::__find_local_by_name_or_abs_path(&wide_string)
     }
     pub(crate) fn __find_local_by_name_or_abs_path(
         module: &U16CStr,
-    ) -> Result<Option<Self>, Win32OrNulError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         let handle = unsafe { GetModuleHandleW(module.as_ptr()) };
         if handle.is_null() {
-            let err = Win32Error::new();
-            if err.get_error_code() == ERROR_MOD_NOT_FOUND {
+            let err = Win32Error::get_last_error();
+            if err.code() == ERROR_MOD_NOT_FOUND {
                 return Ok(None);
             }
 
             return Err(err.into());
         }
 
-        Ok(Some(unsafe { Self::new_local(handle) }))
+        Ok(Some(unsafe { Self::new_local_unchecked(handle) }))
     }
 
     /// Searches for a module with the given name in the given process.
@@ -139,7 +140,7 @@ impl<'a> ProcessModule<'a> {
     fn _find_remote_by_name(
         module_name: impl AsRef<Path>,
         process: ProcessRef<'a>,
-    ) -> Result<Option<ProcessModule<'a>>, Win32OrNulError> {
+    ) -> Result<Option<ProcessModule<'a>>, IoOrNulError> {
         assert!(!process.is_current());
 
         process
@@ -151,7 +152,7 @@ impl<'a> ProcessModule<'a> {
     fn _find_remote_by_path(
         module_path: impl AsRef<Path>,
         process: ProcessRef<'a>,
-    ) -> Result<Option<Self>, FindModuleByPathError> {
+    ) -> Result<Option<Self>, IoOrNulError> {
         assert!(!process.is_current());
 
         process
@@ -162,7 +163,7 @@ impl<'a> ProcessModule<'a> {
     /// Gets the underlying handle to the module.
     #[must_use]
     pub const fn handle(&self) -> ModuleHandle {
-        self.handle
+        self.handle.as_ptr()
     }
     /// Gets the process this module belongs to.
     #[must_use]
@@ -202,7 +203,7 @@ impl<'a> ProcessModule<'a> {
             )
         };
         if result == 0 {
-            return Err(Win32Error::new());
+            return Err(Win32Error::get_last_error());
         }
 
         let module_path_len = result as usize;
@@ -223,7 +224,7 @@ impl<'a> ProcessModule<'a> {
             )
         };
         if result == 0 {
-            return Err(Win32Error::new());
+            return Err(Win32Error::get_last_error());
         }
 
         let module_path_len = result as usize;
@@ -259,7 +260,7 @@ impl<'a> ProcessModule<'a> {
             )
         };
         if result == 0 {
-            return Err(Win32Error::new());
+            return Err(Win32Error::get_last_error());
         }
 
         let module_name_len = result as usize;
@@ -291,7 +292,7 @@ impl<'a> ProcessModule<'a> {
 
         let fn_ptr = unsafe { GetProcAddress(self.handle(), proc_name.as_ptr()) };
         if fn_ptr.is_null() {
-            return Err(Win32Error::new());
+            return Err(Win32Error::get_last_error());
         }
         Ok(fn_ptr)
     }
