@@ -1,11 +1,11 @@
 mod array_or_vec;
 pub(crate) use array_or_vec::*;
 
-mod foreign_process_memory;
-pub(crate) use foreign_process_memory::*;
-
+#[allow(dead_code)]
 mod retry;
+#[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
 pub(crate) use retry::*;
 
+#[allow(dead_code)]
 mod uninit_buf;
 pub(crate) use uninit_buf::*;
